@@ -1,10 +1,10 @@
 #' Parsed sourced file from a filename
 #'
-#' This object is given as input to each linter
+#' This object is given as input to each linter.
 #'
 #' @details
-#' The file is read in using the `encoding` setting.
-#' This setting found by taking the first valid result from the following locations
+#' The file is read using the `encoding` setting.
+#' This setting is found by taking the first valid result from the following locations
 #'
 #' 1. The `encoding` key from the usual lintr configuration settings.
 #' 2. The `Encoding` field from a Package `DESCRIPTION` file in a parent directory.
@@ -39,7 +39,7 @@
 #'     \item{`filename` (`character`)}
 #'     \item{`file_lines` (`character`) the [readLines()] output for this file}
 #'     \item{`content` (`character`) for .R files, the same as `file_lines`;
-#'           for .Rmd scripts, this is the extracted R source code (as text)}
+#'           for .Rmd or .qmd scripts, this is the extracted R source code (as text)}
 #'     \item{`full_parsed_content` (`data.frame`) as given by
 #'           [utils::getParseData()] for the full content}
 #'     \item{`full_xml_parsed_content` (`xml_document`) the XML parse tree of all
@@ -51,8 +51,11 @@
 #'   \item{error}{A `Lint` object describing any parsing error.}
 #'   \item{lines}{The [readLines()] output for this file.}
 #' }
+#'
+#' @examplesIf requireNamespace("withr", quietly = TRUE)
+#' tmp <- withr::local_tempfile(lines = c("x <- 1", "y <- x + 1"))
+#' get_source_expressions(tmp)
 #' @export
-#' @md
 get_source_expressions <- function(filename, lines = NULL) {
   source_expression <- srcfile(filename, encoding = settings$encoding)
 
@@ -79,12 +82,12 @@ get_source_expressions <- function(filename, lines = NULL) {
   names(source_expression$lines) <- seq_along(source_expression$lines)
   source_expression$content <- get_content(source_expression$lines)
   parsed_content <- get_source_expression(source_expression, error = function(e) lint_parse_error(e, source_expression))
-  top_level_map <- generate_top_level_map(parsed_content)
 
-  if (inherits(e, "lint") && !nzchar(e$line)) {
+  if (inherits(e, "lint") && (is.na(e$line) || !nzchar(e$line) || e$message == "unexpected end of input")) {
     # Don't create expression list if it's unreliable (invalid encoding or unhandled parse error)
     expressions <- list()
   } else {
+    top_level_map <- generate_top_level_map(parsed_content)
     xml_parsed_content <- safe_parse_to_xml(parsed_content)
 
     expressions <- lapply(
@@ -107,15 +110,14 @@ get_source_expressions <- function(filename, lines = NULL) {
     }
 
     # add global expression
-    expressions[[length(expressions) + 1L]] <-
-      list(
-        filename = filename,
-        file_lines = source_expression$lines,
-        content = source_expression$lines,
-        full_parsed_content = parsed_content,
-        full_xml_parsed_content = xml_parsed_content,
-        terminal_newline = terminal_newline
-      )
+    expressions[[length(expressions) + 1L]] <- list(
+      filename = filename,
+      file_lines = source_expression$lines,
+      content = source_expression$lines,
+      full_parsed_content = parsed_content,
+      full_xml_parsed_content = xml_parsed_content,
+      terminal_newline = terminal_newline
+    )
   }
 
   list(expressions = expressions, error = e, lines = source_expression$lines)
@@ -150,6 +152,30 @@ lint_parse_error <- function(e, source_expression) {
   lint_parse_error_nonstandard(e, source_expression)
 }
 
+#' Ensure a string is valid for printing
+#'
+#' Helper to ensure a valid string is provided as line where necessary.
+#' Handles two cases:
+#'
+#'  1. NA lines, as generated by running outside of code blocks in Rmd documents
+#'  2. invalid string lines, as generated by invalid encoding settings
+#'
+#' [nchar()] can detect both cases: 1. returns `NA_integer_`, 2. errors
+#'
+#' @param line Possibly misencoded or NA line
+#'
+#' @return `line` if it's a valid non-NA string, otherwise an empty string.
+#'
+#' @noRd
+fixup_line <- function(line) {
+  nchars <- tryCatch(nchar(line, type = "chars"), error = function(e) NA_integer_)
+  if (is.na(nchars)) {
+    ""
+  } else {
+    line
+  }
+}
+
 #' Convert an R >= 4.3.0 classed parseError with metadata into a lint
 #'
 #' @param e A parse error of class `parseError` generated by R >= 4.3.0
@@ -160,7 +186,7 @@ lint_parse_error <- function(e, source_expression) {
 #' @noRd
 lint_parse_error_r43 <- function(e, source_expression) {
   msg <- rex::re_substitutes(e$message, rex::rex(" (", except_some_of(")"), ")", end), "")
-  line <- e$lineno
+  line_number <- e$lineno
   column <- e$colno
   substr(msg, 1L, 1L) <- toupper(substr(msg, 1L, 1L))
   msg <- paste0(msg, ".")
@@ -170,23 +196,27 @@ lint_parse_error_r43 <- function(e, source_expression) {
   }
 
   if (column == 0L) {
-    line <- line - 1L
-    column <- nchar(source_expression$lines[[line]])
+    line_number <- line_number - 1L
   }
 
-  if (line < 1L || line > length(source_expression$lines)) {
+  if (line_number < 1L || line_number > length(source_expression$lines)) {
     # Safely handle invalid location info
-    line <- 1L
+    line_number <- 1L
     column <- 1L
   }
 
+  line <- fixup_line(source_expression$lines[[line_number]])
+  if (column == 0L || column > nchar(line) + 1L) {
+    column <- nchar(line) + 1L
+  }
+
   Lint(
     filename = source_expression$filename,
-    line_number = line,
+    line_number = line_number,
     column_number = column,
     type = "error",
     message = msg,
-    line = source_expression$lines[[line]]
+    line = line
   )
 }
 
@@ -206,10 +236,10 @@ lint_parse_error_r42 <- function(message_info, source_expression) {
   # end of the previous line
   if (column_number %==% 0L) {
     line_number <- line_number - 1L
-    line <- source_expression$lines[[line_number]]
-    column_number <- nchar(line)
+    line <- fixup_line(source_expression$lines[[line_number]])
+    column_number <- nchar(line) + 1L
   } else {
-    line <- source_expression$lines[[line_number]]
+    line <- fixup_line(source_expression$lines[[line_number]])
   }
 
   Lint(
@@ -247,7 +277,7 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
         column_number = 1L,
         type = "error",
         message = "Invalid multibyte character in parser. Is the encoding correct?",
-        line = source_expression$lines[[l]]
+        line = fixup_line(source_expression$lines[[l]])
       )
     )
   } else if (grepl("invalid multibyte string, element", e$message, fixed = TRUE)) {
@@ -265,10 +295,11 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
   } else if (grepl("repeated formal argument", e$message, fixed = TRUE)) {
     matches <- re_matches(
       e$message,
-      rex("repeated formal argument '",
-          capture(name = "symbol", anything),
-          "' on line ",
-          capture(name = "line", digits)
+      rex(
+        "repeated formal argument '",
+        capture(name = "symbol", anything),
+        "' on line ",
+        capture(name = "line", digits)
       )
     )
     sym <- matches$symbol
@@ -281,7 +312,7 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
         column_number = 1L,
         type = "error",
         message = sprintf("Repeated formal argument '%s'.", sym),
-        line = source_expression$lines[[l]]
+        line = fixup_line(source_expression$lines[[l]])
       )
     )
   }
@@ -336,10 +367,14 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
     )
   }
 
-  message_info <- re_matches(e$message,
-                             rex(single_quotes, capture(name = "name", anything), single_quotes,
-                                 anything,
-                                 double_quotes, capture(name = "starting", anything), double_quotes))
+  message_info <- re_matches(
+    e$message,
+    rex(
+      single_quotes, capture(name = "name", anything), single_quotes,
+      anything,
+      double_quotes, capture(name = "starting", anything), double_quotes
+    )
+  )
 
   loc <- re_matches(source_expression$content, rex(message_info$starting), locations = TRUE)
   line_location <- loc[!is.na(loc$start) & !is.na(loc$end), ]
@@ -347,12 +382,16 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
   if (nrow(line_location) == 0L) {
     if (grepl("attempt to use zero-length variable name", e$message, fixed = TRUE)) {
       # empty symbol: ``, ``(), ''(), ""(), fun(''=42), fun(""=42), fun(a=1,""=42)
-      loc <- re_matches(source_expression$content,
-                        rex("``" %or%
-                              list(or("''", '""'), any_spaces, "(") %or%
-                              list(or("(", ","), any_spaces, or("''", '""'), any_spaces, "=")),
-                        options = "multi-line",
-                        locations = TRUE)
+      loc <- re_matches(
+        source_expression$content,
+        rex(
+          "``" %or%
+            list(or("''", '""'), any_spaces, "(") %or%
+            list(or("(", ","), any_spaces, or("''", '""'), any_spaces, "=")
+        ),
+        options = "multi-line",
+        locations = TRUE
+      )
       loc <- loc[!is.na(loc$start) & !is.na(loc$end), ]
       if (nrow(loc) > 0L) {
         line_location <- loc[1L, ]
@@ -388,20 +427,20 @@ lint_parse_error_nonstandard <- function(e, source_expression) {
 }
 
 lint_rmd_error <- function(e, source_expression) {
-  message_info <- re_matches(e$message,
-    rex(except_some_of(":"),
+  message_info <- re_matches(
+    e$message,
+    rex(
+      except_some_of(":"),
       ":",
-      capture(name = "line",
-        digits),
+      capture(name = "line", digits),
       ":",
-      capture(name = "column",
-        digits),
+      capture(name = "column", digits),
       ":",
       space,
-      capture(name = "message",
-        anything),
-      "\n")
+      capture(name = "message", anything),
+      "\n"
     )
+  )
 
   line_number <- as.integer(message_info$line)
   column_number <- as.integer(message_info$column)
@@ -444,8 +483,8 @@ get_single_source_expression <- function(loc,
 get_source_expression <- function(source_expression, error = identity) {
   parse_error <- FALSE
 
-  tryCatch(
-    source_expression$parsed_content <- parse(
+  parsed_content <- tryCatch(
+    parse(
       text = source_expression$content,
       srcfile = source_expression,
       keep.source = TRUE
@@ -453,10 +492,12 @@ get_source_expression <- function(source_expression, error = identity) {
     error = error
   )
 
-  # This needs to be done twice to avoid
+  # TODO: Remove when minimum R version is bumped to > 3.5
+  #
+  # This needs to be done twice to avoid a bug fixed in R 3.4.4
   #   https://bugs.r-project.org/bugzilla/show_bug.cgi?id=16041
-  e <- tryCatch(
-    source_expression$parsed_content <- parse(
+  parsed_content <- tryCatch(
+    parse(
       text = source_expression$content,
       srcfile = source_expression,
       keep.source = TRUE
@@ -464,55 +505,36 @@ get_source_expression <- function(source_expression, error = identity) {
     error = error
   )
 
-  if (inherits(e, c("error", "lint"))) {
-    assign("e", e, envir = parent.frame())
+  if (inherits(parsed_content, c("error", "lint"))) {
+    assign("e", parsed_content, envir = parent.frame())
     parse_error <- TRUE
   }
 
   # Triggers an error if the lines contain invalid characters.
-  e <- tryCatch(
+  parsed_content <- tryCatch(
     nchar(source_expression$content, type = "chars"),
     error = error
   )
 
-  if (inherits(e, c("error", "lint"))) {
+  if (inherits(parsed_content, c("error", "lint"))) {
     # Let parse errors take precedence over encoding problems
-    if (!parse_error) assign("e", e, envir = parent.frame())
+    if (!parse_error) assign("e", parsed_content, envir = parent.frame())
     return() # parsed_content is unreliable if encoding is invalid
   }
 
+  source_expression$parsed_content <- parsed_content
   fix_octal_escapes(fix_eq_assigns(fix_tab_indentations(source_expression)), source_expression$lines)
 }
 
 get_newline_locs <- function(x) {
   newline_search <- re_matches(x, rex("\n"), locations = TRUE, global = TRUE)[[1L]]$start
-  c(0L,
+  c(
+    0L,
     if (!is.na(newline_search[1L])) newline_search,
     nchar(x) + 1L
   )
 }
 
-find_line_fun <- function(content, newline_locs) {
-  function(line_number) {
-    warning(
-      "find_line is deprecated and will soon be removed. ",
-      "XPath logic and xml_nodes_to_lints() are usually preferable"
-    )
-    which(newline_locs >= line_number)[1L] - 1L
-  }
-}
-
-find_column_fun <- function(content, newline_locs) {
-  function(line_number) {
-    warning(
-      "find_column is deprecated and will soon be removed. ",
-      "XPath logic and xml_nodes_to_lints() are usually preferable"
-    )
-    matched_line_number <- which(newline_locs >= line_number)[1L] - 1L
-    line_number - newline_locs[matched_line_number]
-  }
-}
-
 # Fix column numbers when there are tabs
 # getParseData() counts 1 tab as a variable number of spaces instead of one:
 # https://github.com/wch/r-source/blame/e7401b68ab0e032fce3e376aaca9a5431619b2b4/src/main/gram.y#L512
@@ -575,7 +597,7 @@ tab_offsets <- function(tab_columns) {
   vapply(
     tab_columns - 1L,
     function(tab_idx) {
-      offset <- 7L - (tab_idx + cum_offset) %% 8L  # using a tab width of 8 characters
+      offset <- 7L - (tab_idx + cum_offset) %% 8L # using a tab width of 8 characters
       cum_offset <<- cum_offset + offset
       offset
     },
@@ -628,10 +650,12 @@ fix_eq_assigns <- function(pc) {
     #   so it likely requires some GHA print debugging -- tedious :)
     end <- true_locs[i]
     j <- end + 1L
+    # nocov start: only runs on certain R versions
     while (j <= length(expr_locs) && !expr_locs[j]) {
       end <- j
       j <- j + 1L
     }
+    # nocov end
 
     prev_loc <- prev_locs[start]
     next_loc <- next_locs[end]
