@@ -4,10 +4,33 @@
 #'   inside them, i.e., directly following an opening delimiter or directly
 #'   preceding a closing delimiter.
 #'
+#' @examples
+#' # will produce lints
+#' lint(
+#'   text = "c( TRUE, FALSE )",
+#'   linters = spaces_inside_linter()
+#' )
+#'
+#' lint(
+#'   text = "x[ 1L ]",
+#'   linters = spaces_inside_linter()
+#' )
+#'
+#' # okay
+#' lint(
+#'   text = "c(TRUE, FALSE)",
+#'   linters = spaces_inside_linter()
+#' )
+#'
+#' lint(
+#'   text = "x[1L]",
+#'   linters = spaces_inside_linter()
+#' )
+#'
 #' @evalRd rd_tags("spaces_inside_linter")
 #' @seealso
-#'   [linters] for a complete list of linters available in lintr. \cr
-#'   <https://style.tidyverse.org/syntax.html#parentheses>
+#' - [linters] for a complete list of linters available in lintr.
+#' - <https://style.tidyverse.org/syntax.html#parentheses>
 #' @export
 spaces_inside_linter <- function() {
   left_xpath_condition <- "
@@ -15,14 +38,19 @@ spaces_inside_linter <- function() {
     and @end != following-sibling::*[1]/@start - 1
     and @line1 = following-sibling::*[1]/@line1
   "
-  left_xpath <- glue::glue("//OP-LEFT-BRACKET[{left_xpath_condition}] | //OP-LEFT-PAREN[{left_xpath_condition}]")
+  left_xpath <- glue::glue("
+  //OP-LEFT-BRACKET[{left_xpath_condition}]
+  | //LBB[{left_xpath_condition}]
+  | //OP-LEFT-PAREN[{left_xpath_condition}]")
 
   right_xpath_condition <- "
     not(preceding-sibling::*[1][self::OP-COMMA])
     and @start != preceding-sibling::*[1]/@end + 1
     and @line1 = preceding-sibling::*[1]/@line2
   "
-  right_xpath <- glue::glue("//OP-RIGHT-BRACKET[{right_xpath_condition}] | //OP-RIGHT-PAREN[{right_xpath_condition}]")
+  right_xpath <- glue::glue("
+  //OP-RIGHT-BRACKET[{right_xpath_condition}]
+  | //OP-RIGHT-PAREN[{right_xpath_condition} and not(preceding-sibling::*[1][self::EQ_SUB])]")
 
   Linter(function(source_expression) {
     if (!is_lint_level(source_expression, "file")) {
@@ -33,7 +61,7 @@ spaces_inside_linter <- function() {
 
     left_expr <- xml2::xml_find_all(xml, left_xpath)
     left_msg <- ifelse(
-      xml2::xml_text(left_expr) == "[",
+      xml2::xml_text(left_expr) %in% c("[", "[["),
       "Do not place spaces after square brackets.",
       "Do not place spaces after parentheses."
     )
