@@ -10,8 +10,10 @@ test_that("linters", {
 
 test_that("undesirable functions and operators", {
   # non-empty named list of NAs and character strings
-  vars <- list(all_undesirable_functions, default_undesirable_functions,
-               all_undesirable_operators, default_undesirable_operators)
+  vars <- list(
+    all_undesirable_functions, default_undesirable_functions,
+    all_undesirable_operators, default_undesirable_operators
+  )
 
   for (x in vars) {
     expect_type(x, "list")
@@ -30,7 +32,6 @@ test_that("settings", {
   expect_true(all(nzchar(names(default_settings))))
 })
 
-skip_if_not_installed("patrick")
 patrick::with_parameters_test_that(
   "all default linters throw lints with their name on 'default_linter_testcode.R'",
   {
